import tkinter

WM_MOUSEMOVE = "WM_MOUSEMOVE"
WM_LBUTTONDOWN = "WM_LBUTTONDOWN"
WM_LBUTTONUP = "WM_LBUTTONUP"
WM_LBUTTONDBLCLK = "WM_LBUTTONDBLCLK"
WM_RBUTTONDOWN = "WM_RBUTTONDOWN"
WM_RBUTTONUP = "WM_RBUTTONUP"
WM_RBUTTONDBLCLK = "WM_RBUTTONDBLCLK"
WM_MBUTTONDOWN = "WM_MBUTTONDOWN"
WM_MBUTTONUP = "WM_MBUTTONUP"
WM_MBUTTONDBLCLK = "WM_MBUTTONDBLCLK"


WM_EVENT = (WM_MOUSEMOVE,
            WM_LBUTTONDOWN, WM_LBUTTONUP, WM_LBUTTONDBLCLK,
            WM_RBUTTONDOWN, WM_RBUTTONUP, WM_RBUTTONDBLCLK,
            WM_MBUTTONDOWN, WM_MBUTTONUP, WM_MBUTTONDBLCLK)


def download_winico():
    """
    安装所需要的一些文件资源。
    """
    from sys import executable
    from os.path import exists, split
    from time import sleep
    tcl = split(executable)[0] + "\\tcl\\"
    if exists(tcl):
        if not exists(tcl + "Winico64"):
            print("已经检测到tcl目录存在，准备开始下载...")
            sleep(3)
            import requests
            import zipfile
            import tempfile
            print("开始下载")

            def Get_Data():
                Url = "https://xiangqinxi.github.io/Winico/Winico64.zip"
                Response = requests.get(Url)
                return Url, Response.content

            Url, Data = Get_Data()  # data为byte字节

            _Tmp_File = tempfile.TemporaryFile()  # 创建临时文件

            _Tmp_File.write(Data)  # byte字节数据写入临时文件
            # _tmp_file.seek(0)

            ZipFile = zipfile.ZipFile(_Tmp_File, mode='r')
            for Names in ZipFile.namelist():
                File = ZipFile.extract(Names, tcl)  # 解压到zip目录文件下
                print("已经解压完成文件 ： " + "File")
            ZipFile.close()
            print("下载完毕")
        else:
            print("已经检测到下载文件")
    print("准备开始安装")
    sleep(3)
    from os import rename
    import shutil
    shutil.move(tcl + 'Winico64\\pkg', tcl)
    shutil.move(tcl + 'Winico64\\src', tcl)


class DevTray(object):
    def __init__(self, master: tkinter.Tk, text: str = "托盘图标", icon=None, menu_func=None):
        """
        使用tcl扩展winico制作的托盘图标组件。

        :param master: 设置父组件
        :param text: 设置托盘图标上的提示文本，默认是"托盘图标"
        :param icon: 设置托盘图标上的图标图片，默认是winico的"tkchat.ico"
        :param menu_func: 设置托盘图标的事件。返回给你三个参数，event事件名，x为X坐标，y为Y坐标。
        """
        self.master = master
        self.text = text
        if icon is None:
            from os.path import split
            from sys import executable
            self.icon = tcl = split(executable)[0] + "\\tcl\\pkg\\tkchat.ico"
        else:
            self.icon = icon
        if menu_func is None:
            self.menu_func = self.menu
        else:
            self.menu_func = menu_func

    def taskbar(self):
        """
        用于将图标加入托盘，但是需要安装winico，如果没安装，请使用download_winico()安装
        """
        try:
            self.master.tk.call('package', 'require',
                                'Winico')
            icon = self.master.tk.call('winico', 'createfrom',
                                       self.icon)
            self.master.tk.call('winico', 'taskbar', 'add', icon,
                                '-callback', (self.master.register(self.menu), '%m', '%x', '%y'),
                                '-pos', 0,
                                '-text', self.text)
        except:
            download_winico()

    show = taskbar

    def info(self):
        """
        获取组件的托盘ID与信息
        """
        return self.master.tk.call("winico", "info")

    def delete(self, id):
        """
        删除托盘图标
        :param id: 为删除托盘图标的ID，可以使用info()获取到。
        """
        return self.master.tk.call("winico", "delete", id)

    def delete_me(self):
        """
        删除自己
        """
        return self.master.tk.call("winico", "delete", self.info())

    def menu(self, event, x, y):
        """
        内置菜单，用于初始化。
        """
        if event == WM_RBUTTONDOWN:
            menu = tkinter.Menu(tearoff=False)
            menu.add_command(label="退出", command=self.master.quit)
            menu.tk_popup(x, y)


if __name__ == '__main__':
    Window = tkinter.Tk()
    Tray = DevTray(Window)
    Tray.show()
    Window.mainloop()